# Copyright 2009-2010 Canonical Ltd.
#
# This file is part of desktopcouch.
#
#  desktopcouch is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# desktopcouch is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with desktopcouch.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Eric Casteleijn <eric.casteleijn@canonical.com>
#          Mark G. Saye <mark.saye@canonical.com>
#          Stuart Langridge <stuart.langridge@canonical.com>
#          Chad Miller <chad.miller@canonical.com>
#          Vincenzo Di Somma <vincenzo.di.somma@canonical.com>

"""The Desktop Couch Records API."""

from couchdb.client import Server
import desktopcouch
from desktopcouch.records import server_base
from desktopcouch.records.server_base import OAuthSession

def keyring_lookup(ctx=None):
        if ctx is None:
            ctx = desktopcouch.local_files.DEFAULT_CONTEXT
        return desktopcouch.local_files.get_oauth_tokens(ctx) 

class OAuthCapableServer(Server):
    def __init__(self, uri, oauth_tokens=None, ctx=None):
        """Subclass of couchdb.client.Server which creates a custom
           with OAuth capabilities"""
        if ctx is None:
            ctx = desktopcouch.local_files.DEFAULT_CONTEXT
        if oauth_tokens is None:
            oauth_tokens = desktopcouch.local_files.get_oauth_tokens(ctx)
        (consumer_key, consumer_secret, token, token_secret) = (
            oauth_tokens["consumer_key"], oauth_tokens["consumer_secret"], 
            oauth_tokens["token"], oauth_tokens["token_secret"])
        session = OAuthSession(credentials=oauth_tokens)
        super(OAuthCapableServer, self).__init__(url=uri, session=session)


class CouchDatabase(server_base.CouchDatabaseBase):
    """An small records specific abstraction over a couch db database."""
 
    def __init__(self, database, uri=None, record_factory=None, create=False,
                 oauth_tokens=None,
                 ctx=desktopcouch.local_files.DEFAULT_CONTEXT):
        if not uri:
            port = desktopcouch.find_port(ctx=ctx)
            uri = "http://localhost:%s" % port
        if oauth_tokens:
            oauth_session = OAuthSession(credentials=oauth_tokens)
        else:
            oauth_session = OAuthSession(
                    credentials=keyring_lookup(ctx))
        super(CouchDatabase, self).__init__(
                database, uri, record_factory=record_factory, create=create,
                session=oauth_session)
